#!/bin/bash

# This script creates a dynamic user based on the serial number of the current system by matching it to the serials and users specified in the "users.txt" file in the "Resources" folder.
# If no serial is detected (which shouldn't happen) or no user for the current serial is specified in the "users.txt" file, the default user will be created.
# Add this script to the specified script folder and add the "Resources" folder to the scripts folder at the same level as the script.

# IMPORTANT: This script REQUIRES "mkuser" by Pico Mitchell of Free Geek.
# Download the latest version of mkuser at https://mkuser.sh and place it in the "Resources" folder.

# NOTE: the create user will be an admin.

# Default User:
user_full_name='MDS User'
# NOTE: The users Short Name will be automatically generated like macOS does. For example, if the user_full_name is "MDS User", the Short Name will be set to "mdsuser".
user_password='password'
user_uid=601

readonly SCRIPT_VERSION='2022.5.12-1'

PATH='/usr/bin:/bin:/usr/sbin:/sbin'

script_name_and_version="\"$(basename "$0")\" version ${SCRIPT_VERSION} on macOS $(sw_vers -productVersion) $(sw_vers -buildVersion)"

echo "Starting ${script_name_and_version}..."

if [[ -z "$3" ]]; then
	echo 'You must specify 3 args, of which the 3rd one is the target volume. This is the same order for a package installer so the script can be called easily from MDS.'
	exit 1
fi

script_dir="${0%/*}"
mkuser_path="${script_dir}/Resources/mkuser.sh"

if [[ ! -r "${mkuser_path}" ]]; then
	echo -e "The required \"${mkuser_path}\" file does not exist or is not readable.\nDownload the latest version of mkuser at https://mkuser.sh and place it in the \"Resources\" folder."
	exit 1
fi

system_serial="$(/usr/libexec/PlistBuddy -c 'Print 0:IOPlatformSerialNumber' /dev/stdin <<< "$(ioreg -arc IOPlatformExpertDevice -k IOPlatformSerialNumber -d 1)" 2> /dev/null)"

users_file="${script_dir}/Resources/users.txt"

if [[ -z "${system_serial}" ]]; then
	echo "System serial number not detected. Default \"${user_full_name}\" user will be created."
elif [[ ! -r "${users_file}" ]]; then
	echo "The \"${users_file}\" file does not exist or is not readable. Default \"${user_full_name}\" user will be created."
else
	found_user_for_serial=false

	while IFS=':' read -r this_serial this_name this_password this_uid; do
		if [[ "${system_serial}" == "$(echo "${this_serial}" | tr '[:lower:]' '[:upper:]')" ]]; then
			user_full_name="${this_name}"
			user_password="${this_password}"
			user_uid="${this_uid}"
			found_user_for_serial=true
			break
		fi
	done < "${users_file}"

	if $found_user_for_serial;  then
		echo "Detected dynamic user \"${user_full_name}\" for serial \"${system_serial}\"."
	else
		echo "No dynamic user detected for serial \"${system_serial}\". Default \"${user_full_name}\" user will be created."
	fi
fi

rm -f "${users_file}"

chmod +x "${mkuser_path}"
echo "${user_password}" | "${mkuser_path}" --full-name "${user_full_name}" --uid "${user_uid}" --stdin-password --admin

echo "Completed ${script_name_and_version}"
