#!/bin/sh

#
# Created by Pico Mitchell on 3/31/22
#
# MIT License
#
# Copyright (c) 2022 Pico Mitchell
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

readonly SCRIPT_VERSION='2022.4.4-1'

PATH='/usr/bin:/bin:/usr/sbin:/sbin'

script_name_and_version="\"$(basename "$0")\" version ${SCRIPT_VERSION} on macOS $(sw_vers -productVersion) $(sw_vers -buildVersion)"

echo "Starting ${script_name_and_version}..."

set_desktop_picture_launch_agent_name='com.twocanoes.mds-set-desktop-picture'

set_desktop_picture_launch_agent_name="$(echo "${set_desktop_picture_launch_agent_name}" | tr -cd '[:alnum:]_.-')" # Make sure set_desktop_picture_launch_agent_name doesn't contain any invalid characters if it's manually modified.

if [ -z "${set_desktop_picture_launch_agent_name}" ]; then
	echo "ERROR: Invalid LaunchAgent name specified."
	exit 1
fi

script_resources_folder="${0%/*}/Resources"
desktop_picture_path="${script_resources_folder}/MDSDesktop.jpg"

file_is_image() {
	[ -f "$1" ] && file -bI "$1" 2> /dev/null | grep -q '^image/'
}

if ! file_is_image "${desktop_picture_path}"; then
	for this_resources_file in "${script_resources_folder}/"*; do
		if file_is_image "${this_resources_file}"; then
			desktop_picture_path="${this_resources_file}"
			break
		fi
	done
fi

if ! file_is_image "${desktop_picture_path}"; then
	echo "ERROR: No custom desktop picture found in \"${script_resources_folder}\"."
	exit 2
fi

desktop_picture_folder='/Library/Desktop Pictures'
copied_desktop_picture_path="${desktop_picture_folder}/$(basename "${desktop_picture_path}")"

echo "Copying custom desktop picture \"${desktop_picture_path}\" to \"${3%/}${copied_desktop_picture_path}\"..."

ditto "${desktop_picture_path}" "${3%/}${copied_desktop_picture_path}"

if ! file_is_image "${3%/}${copied_desktop_picture_path}"; then
	echo "ERROR: Failed to copy custom desktop picture to \"${3%/}${copied_desktop_picture_path}\"."
	exit 3
fi

chmod 755 "${3%/}${desktop_picture_folder}"
chmod 644 "${3%/}${copied_desktop_picture_path}"

# The following JavaScript for Automation (JXA) code within the "set_desktop_picture_jxa_code" variable will be set as a string value within the "ProgramArguments" array of a LaunchAgent plist.
# The way the value will be set using "plutil" means that no special xml/plist characters need to be escaped in advance (such as "&" and "<") since "plutil" will take care of that for us.
# This JXA code will be executed by "/usr/bin/osascript -l JavaScript" (also specified in the "ProgramArguments" array of a LaunchAgent plist) and it will set the specified
# desktop picture by retrieving the "DESKTOP_PICTURE_PATH" environment variable set within the LaunchAgent plist (which contains the path to the specified desktop picture).
# Retrieving the desktop picture path from the plist in this way rather than hardcoding the path as a string value into the JXA code
# means that no special characters that could exist in a valid filename need to be escaped manually in advance to not break the JXA code.
# After the desktop picture has been set, this code will then delete the LaunchAgent plist so that nothing is leftover after its job is done.
set_desktop_picture_jxa_code="
ObjC.import('AppKit')
const fileManager = $.NSFileManager.defaultManager
const desktopPicturePath = $.NSProcessInfo.processInfo.environment.objectForKey('DESKTOP_PICTURE_PATH')
if (!desktopPicturePath.isNil() && fileManager.fileExistsAtPath(desktopPicturePath)) {
	const desktopPictureURL = $.NSURL.fileURLWithPath(desktopPicturePath)
	const sharedWorkspace = $.NSWorkspace.sharedWorkspace
	const allScreens = $.NSScreen.screens
	for (let thisScreenIndex = 0; thisScreenIndex < allScreens.count; thisScreenIndex ++) {
		sharedWorkspace.setDesktopImageURLForScreenOptionsError(desktopPictureURL, allScreens.objectAtIndex(thisScreenIndex), ObjC.wrap(), ObjC.wrap())
	}
}
fileManager.removeItemAtPathError(ObjC.wrap('~/Library/LaunchAgents/${set_desktop_picture_launch_agent_name}.plist').stringByExpandingTildeInPath, ObjC.wrap())
"

for this_home_folder in "${3%/}/Users/"* "${3%/}/System/Library/User Template/Non_localized"; do
	if [ -n "${this_home_folder}" ] && [ -d "${this_home_folder}/Library" ] && [ "${this_home_folder}" != "${3%/}/Users/Shared" ] && [ "${this_home_folder}" != "${3%/}/Users/Guest" ]; then
		echo "Creating \"${set_desktop_picture_launch_agent_name}\" LaunchAgent in \"${this_home_folder}\"..."

		this_user_launch_agents_folder="${this_home_folder}/Library/LaunchAgents"
		if [ ! -d "${this_user_launch_agents_folder}" ]; then
			mkdir "${this_user_launch_agents_folder}"
			chmod 755 "${this_user_launch_agents_folder}"
		fi

		this_user_launch_agent_path="${this_user_launch_agents_folder}/${set_desktop_picture_launch_agent_name}.plist"
		rm -rf "${this_user_launch_agent_path}" # This LaunchAgent plist path shouldn't already exist, but delete it anyway just in case (even if somehow it's a folder).

		# Use "PlistBuddy" for the inital LaunchAgent plist creation since it will create non-existent plists automatically (unlike "plutil"), and can add multiple values in a single invocation (unlike "plutil").
		# NOTE: "plutil" now has the "-create" option to create a new plist, but that is only available on macOS 12 Monterey and newer.
		/usr/libexec/PlistBuddy \
			-c "Add :Label string ${set_desktop_picture_launch_agent_name}" \
			-c 'Add :ProgramArguments array' \
			-c 'Add :ProgramArguments: string /usr/bin/osascript' \
			-c 'Add :ProgramArguments: string -l' \
			-c 'Add :ProgramArguments: string JavaScript' \
			-c 'Add :ProgramArguments: string -e' \
			-c 'Add :EnvironmentVariables dict' \
			-c 'Add :RunAtLoad bool true' \
			-c 'Add :StandardOutPath string /dev/null' \
			-c 'Add :StandardErrorPath string /dev/null' \
			"${this_user_launch_agent_path}"

		# Now, use "plutil" for the values that may contain special xml/plist characters that need to be escaped.
		# "PlistBuddy" would escape special xml/plist characters in values properly too, but because of how "PlistBuddy" values need to be specified inside of commands that
		# are within a quoted string means that quotes in the values (which could also exist) would need to be escaped to not break the "PlistBuddy" commands themselves.
		# That possible nested quoting issue doesn't exist with "plutil" because of how values are specified as their own separate argument rather than within commands like with "PlistBuddy".

		plutil -insert 'ProgramArguments.4' -string "${set_desktop_picture_jxa_code}" "${this_user_launch_agent_path}" # See comments about this code above when the "set_desktop_picture_jxa_code" variable is set.
		# NOTE: Use dot separated "keypath" notation with the index to insert the string into the existing array INSTEAD OF using the "-append" option since that is only available on macOS 12 Monterey and newer.

		plutil -insert 'EnvironmentVariables.DESKTOP_PICTURE_PATH' -string "${copied_desktop_picture_path}" "${this_user_launch_agent_path}"
		# The "DESKTOP_PICTURE_PATH" environment variable will be retrieved by the "set_desktop_picture_jxa_code" script that will be run by the LaunchAgent.
		# Having this path set in the plist by "plutil" means that any characters (such as "&", "<", or ">", etc) that could exist in a valid filename and would break xml/plist syntax will be properly escaped by "plutil" automatically.
		# Also, the way that this value will be retreived using ObjC methods via JXA means that no other characters that could break JXA code (such as quotes or backslashes) need to be escaped either.

		chmod 644 "${this_user_launch_agent_path}"

		this_username_for_home_folder="$(dscl . -search /Users NFSHomeDirectory "${this_home_folder}" 2> /dev/null | awk '{ print $1; exit }')"
		if [ -n "${this_username_for_home_folder}" ]; then
			# The username is needed to "chown" the LaunchAgent folder and file created in this script by the root user.
			# No username will be set when editing the "User Template" folder, but that's ok since the permissions will be set properly by macOS when a new user is created.

			chown -R "${this_username_for_home_folder}" "${this_user_launch_agents_folder}"
		fi
	fi
done

echo "Completed ${script_name_and_version}"
